-- StubbleSowingMachine
-- Spec for StubbleSowingMachine to be used with chopped straw
-- When activated, the sowingMachine will work the soil but let the choppedStraw layer to have distinct visible behavior between Stubble SowingMachine and Standard SowingMachine
-- by yumi

source(Utils.getFilename("StubbleSowingMachineEvent.lua", g_currentModDirectory))

StubbleSowingMachine = {};

function StubbleSowingMachine.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(StubbleSowingMachine, specializations);
end;

function StubbleSowingMachine:load(xmlFile)
  self.setStubbleSowingMachineActive = SpecializationUtil.callSpecializationsFunction("setStubbleSowingMachineActive");
  self.isStubbleSowingMachineActive = false;
end;

function StubbleSowingMachine:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
  if not resetVehicles then
    local isStubbleSowingMachineActive= Utils.getNoNil(getXMLBool(xmlFile, key.."#StubbleSowingMachine"),false);

    if isStubbleSowingMachineActive == nil then
      self:setStubbleSowingMachineActive(false);
      --self.isStubbleSowingMachineActive = false;
    else
      self:setStubbleSowingMachineActive(isStubbleSowingMachineActive);
      --self.isStubbleSowingMachineActive = isStubbleSowingMachineActive;
    end

  end;
  return BaseMission.VEHICLE_LOAD_OK;
end;
 
function StubbleSowingMachine:getSaveAttributesAndNodes(nodeIdent)
  local attributes = 'StubbleSowingMachine="'.. tostring(self.isStubbleSowingMachineActive) .. '"';
  return attributes, nil;
end;

function StubbleSowingMachine:delete()
end;

function StubbleSowingMachine:readStream(streamId, connection)
  self:setStubbleSowingMachineActive(streamReadBool(streamId), true);
end;

function StubbleSowingMachine:writeStream(streamId, connection)
  streamWriteBool(streamId, self.isStubbleSowingMachineActive);
end;

function StubbleSowingMachine:mouseEvent(posX, posY, isDown, isUp, button)

end;

function StubbleSowingMachine:keyEvent(unicode, sym, modifier, isDown)
end;

function StubbleSowingMachine:onLeave()

end

function StubbleSowingMachine:update(dt)
  if self:getIsActiveForInput() and self.isClient then
    if InputBinding.hasEvent(InputBinding.OnOffStubbleSowingMachine) then
      self:setStubbleSowingMachineActive(not self.isStubbleSowingMachineActive);
    end
  end
end;

function StubbleSowingMachine:updateTick(dt)

  local preparingOutputId = nil;
  
  if self.sowingMachineHasGroundContact and self:isLowered() then
    if g_currentMission.fruits[FruitUtil.FRUITTYPE_CHOPPEDSTRAW] and g_currentMission.fruits[FruitUtil.FRUITTYPE_CHOPPEDRAPE] and g_currentMission.fruits[FruitUtil.FRUITTYPE_CHOPPEDMAIZE] and self.isStubbleSowingMachineActive then
      local fruitIdx = nil;
      
      local cx,cz;
      cx,_,cz = getWorldTranslation(self.rootNode);

      if cx ~= nil and cx==cx and cz==cz then -- Make extra sure that there actually is an x,z coordinate to use.
        local squareSize = 10
        local widthX,widthZ, heightX,heightZ = squareSize-0.5,0, 0,squareSize-0.5
        local x,z = cx - (squareSize/2), cz - (squareSize/2)
          
        -- Get current fruit on the groud
        local foundFruits = nil;
        for fruitIndex,fruit in pairs(g_currentMission.fruits) do
          if fruit.id ~= nil and fruit.id ~= 0 then

            setDensityCompareParams(fruit.id, "between", 2, 10)  -- #fruit from 1st growing state to withered state
            local _,numPixels,_ = getDensityParallelogram(fruit.id, x,z, widthX,widthZ, heightX,heightZ, 0,g_currentMission.numFruitStateChannels)
            setDensityCompareParams(fruit.id, "greater", 0)

            if numPixels > 0 then
              local fillTypeIndex = FruitUtil.fruitTypeToFillType[fruitIndex]
              local fillTypeName = Fillable.fillTypeIndexToDesc[fillTypeIndex].nameI18N
              if fillTypeName == nil then
                fillTypeName = Fillable.fillTypeIndexToDesc[fillTypeIndex].name
                if g_i18n:hasText(fillTypeName) then
                  fillTypeName = g_i18n:getText(fillTypeName)
                end
              end
              foundFruits = ((foundFruits == nil) and "" or foundFruits..", ") .. fillTypeName
              --print("We have found some " .. fillTypeName);
              --print("numPixels: " .. tostring(numPixels > 0) .. " numPixels1: " .. tostring(numPixels1 > 0))
              -- keep 1st one for now
              if fruitIdx == nil then
                fruitIdx = fruitIndex;
                break;
              end
            end
          end
        end
      end            
      
      if fruitIdx ~= nil then
        local fruitDesc = FruitUtil.fruitIndexToDesc[fruitIdx];
        if fruitDesc.name == "maize" or fruitDesc.name == "cornseed" then
          preparingOutputId = g_currentMission.fruits[FruitUtil.FRUITTYPE_CHOPPEDMAIZE].preparingOutputId;
          --print("FRUITTYPE_CHOPPEDMAIZE");
        elseif fruitDesc.name == "rape" or fruitDesc.name == "sunflower" then
          preparingOutputId = g_currentMission.fruits[FruitUtil.FRUITTYPE_CHOPPEDRAPE].preparingOutputId;
          --print("FRUITTYPE_CHOPPEDRAPE");
        elseif fruitDesc.name ~= "rape" and fruitDesc.name ~= "maize" and fruitDesc.name ~= "sunflower" and fruitDesc.name ~= "cornseed" then
          preparingOutputId = g_currentMission.fruits[FruitUtil.FRUITTYPE_CHOPPEDSTRAW].preparingOutputId;      
          --print("FRUITTYPE_CHOPPEDSTRAW");
        end

        for _,cuttingArea in pairs(self.workAreas) do
          local x, _, z = getWorldTranslation(cuttingArea.start)
          local x1, _, z1 = getWorldTranslation(cuttingArea.width)
          local x2, _, z2 = getWorldTranslation(cuttingArea.height)
          
          Utils.updateStrawHaulmArea(preparingOutputId, x, z, x1, z1, x2, z2)
        end
      end;
    end
  end
end;

function StubbleSowingMachine:draw()
		--display a help message to say how to desactivate the lock
    if self.isStubbleSowingMachineActive then
      g_currentMission:addHelpButtonText(StubbleSowingMachine.l10nTexts["StubbleSowingMachine_Deactivate"], InputBinding.OnOffStubbleSowingMachine); --g_i18n:getText("StubbleSowingMachine_Deactivate"), InputBinding.OnOff);
    else
      g_currentMission:addHelpButtonText(StubbleSowingMachine.l10nTexts["StubbleSowingMachine_Activate"], InputBinding.OnOffStubbleSowingMachine); --g_i18n:getText("StubbleSowingMachine_Activate"), InputBinding.OnOff);
    end
end;

function StubbleSowingMachine:attachImplement(implement)
end;

function StubbleSowingMachine:setStubbleSowingMachineActive(isActive, noEventSend)
  self.isStubbleSowingMachineActive = isActive;
  if noEventSend ~= true then
    StubbleSowingMachineEvent.sendEvent(self, isActive);
  end;
end;
